#include <stdio.h>
#include <util/delay.h>
#include <avr/io.h>
#include "SevenSeg.h"
#include "bool.h"
#include "trtSettings.h"
#include "trtLib.h"

//D0 -> 7-seg pin 14
//D1 -> 7-seg pin 16
//D2 -> 7-seg pin 13
//D3 -> 7-seg pin 3
//D4 -> 7-seg pin 5
//D5 -> 7-seg pin 11
//D6 -> 7-seg pin 15
//D7 -> 7-seg pin 7

unsigned char digit0 = 0, digit1 = 0, digit2 = 0, digit3 = 0;
bool upperDot = false, colon = false;

unsigned char digitTable[11] = {
0b11111111,//null
0b11000000,//0
0b11111001,//1
0b10100100,//2
0b10110000,//3
0b10011001,//4
0b10010010,//5
0b10000010,//6
0b11111000,//7
0b10000000,//8
0b10010000//9
};

void seven_seg_setDigits(char d3, char d2, char d1, char d0) {
	digit0 = d0 + 1;
	digit1 = d1 + 1;
	digit2 = d2 + 1;
	digit3 = d3 + 1;
}

void seven_seg_setColon(bool col) {
	if (!colon && col)
		PORTC &= 0b00010000;
	else if (colon && !col)
		PORTC |= 0b11101111;
	colon = col;
}

void seven_seg_toggleColon() {
	if (colon)
		PORTC |= 0b00010000;
	else PORTC &= 0b11101111;
	colon = !colon;
}

void seven_seg_setUpperDot(bool ud) {
	if (!upperDot && ud)
		PORTC |= 0b00100000;
	else if (upperDot && !ud)
		PORTC &= 0b11011111;
	upperDot = ud;
}

void seven_seg_init() {
	DDRD = 0b11111111;
	PORTD = 0b11111111;
	DDRC = (DDRC | 0b00111111); // use low 6 bits of C as output
	PORTC = PORTC & 0b11000000; // set them low
}

void seven_seg_run() {
	uint32_t rel, dead;
	while (1) {
		PORTC = PORTC | 0b00000001; //drive digit 0
		PORTD = digitTable[digit0];
		_delay_us(100);
		PORTC = PORTC & 0b11111110;
		_delay_us(100);

		PORTC = PORTC | 0b00000010; //drive digit 1
		PORTD = digitTable[digit1];
		_delay_us(100);
		PORTC = PORTC & 0b11111101;
		_delay_us(100);

		PORTC = PORTC | 0b00000100; //drive digit 2
		PORTD = digitTable[digit2];
		_delay_us(100);
		PORTC = PORTC & 0b11111011;
		_delay_us(100);

		PORTC = PORTC | 0b00001000; //drive digit 3
		PORTD = digitTable[digit3];
		_delay_us(100);
		PORTC = PORTC & 0b11110111;
		_delay_us(100);

		rel = trtCurrentTime() + SECONDS2TICKS(0.001);
    	dead = trtCurrentTime() + SECONDS2TICKS(0.002);
    	trtSleepUntil(rel, dead);
	}
}
